ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c
c  iccg subroutine solving  xout=a^(-1) b
c
c  a: symmetric sparse matrix, with only 11 (=5*2+1) non-zero diagnal
c     lines, of dimension nx*ny*nz, a(i,j,k,1) is the diagnal term;
c     a(i,j,k,2) is the coefficient associated with k+1, k-1;
c     a(i,j,k,3) is the coefficient associated with j+1, j-1;
c     a(i,j,k,4) is the coefficient associated with i+1, i-1;
c     a5=a(i,j,k,5)=terms needed for periodic y BC
c     a6=a(i,j,k,6)=terms needed for periodic x BC
c  b: known vector
c  xout: unknown to be solved
c  mx,my,mz: grid numbers in x,y,z
c   ix=0,mx+1; jx=0,my+1; kx=0,mz+1 are boundary terms
c  eps_i: allowed deviation 
c  eps_o: deviation of the result
c  itr_i: allowed maximum iteration
c  itr_o iteration step
c  ier=0 if the result is within allowed deviation, otherwise ier=1
c  
C**********************************************************************
C   update log
C   Jan. 23, 1999  decomp.f is called only once in the main program
C   Dec. 27, 1999  decomp.f seperated
C   Dec.  2, 1998  MPI version
C   Nov. 18, 1998  vectorize for induction solver
C   Oct. 16, 1998  for ICCG of periodic BC
C   Sep.  1, 1998  coded by W. Chou, based on the example in IBM manual
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      subroutine iccg(b,xout,mx,my,mz,eps_i,eps_o,itr_i,itr_o,ier)
      include 'common.grid'
      include 'common.mat'
      include 'common.para'
      COMMON /CMSTEP/ DT,TIME,NS,NSFILE
      dimension b(0:nx,0:ny,0:nz),xout(0:nx,0:ny,0:nz)
      dimension p(0:nx,0:ny,0:nz)
      dimension q(0:nx,0:ny,0:nz),r(0:nx,0:ny,0:nz)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c   initialize all variables, be careful not to change the boundary
c   of xout
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      do k=0,mz+1
      do j=0,my+1
      do i=0,mx+1
         p(i,j,k)=0.0
         q(i,j,k)=0.0
         r(i,j,k)=0.0
      enddo
      enddo
      enddo
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c    q=a x_0, r_0=b-q, p_0=(LDLt)^(-1) r_0, c1=(r_0,p_0)
c     start parallelization from now on
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      call amultx(smain,xout,q,mx,my,mz)
      do k=ksta,kend
      do j=1,my
      do i=1,mx
         r(i,j,k)=b(i,j,k)-q(i,j,k)
      enddo
      enddo
      enddo
      call ldlt(r,p,mx,my,mz)
      cc1=0.0
      do k=ksta,kend
      do j=1,my
      do i=1,mx
        cc1=cc1+r(i,j,k)*p(i,j,k)
      enddo
      enddo
      enddo
      call mpi_allreduce(cc1,c1,1,mpi_real,mpi_sum,mpi_comm_world,ierr)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     iteration starts here
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      do 10000 iteration=1,itr_i
        call amultx(smain,p,q,mx,my,mz)
        cc2=0.0
        do k=ksta,kend
        do j=1,my
        do i=1,mx
           cc2=cc2+p(i,j,k)*q(i,j,k)
        enddo
        enddo
        enddo
        call mpi_allreduce(cc2,c2,1,mpi_real,mpi_sum,
     &                     mpi_comm_world,ierr)
        alpha=c1/c2
        xx1=0.0
        xx2=0.0
        do k=ksta,kend
        do j=1,my
        do i=1,mx
           xold=xout(i,j,k)
           xout(i,j,k)=xout(i,j,k)+alpha*p(i,j,k)
           r(i,j,k)=r(i,j,k)-alpha*q(i,j,k)
           xx1=xx1+xold*xold
           xx2=xx2+(xout(i,j,k)-xold)**2
        enddo
        enddo
        enddo
        call mpi_allreduce(xx1,x1,1,mpi_real,mpi_sum,
     &                     mpi_comm_world,ierr)
        call mpi_allreduce(xx2,x2,1,mpi_real,mpi_sum,
     &                     mpi_comm_world,ierr)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     if sqrt(x2/x1) small than desired eps_i, then end iteration 
c     and return
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
        if (x1 .ne. 0.0) then
           res=sqrt(x2/x1)
           if (res .le. eps_i) then
             itr_o=iteration
             eps_o=res
             ier=0
             return
           endif
        endif
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     not accurate enough, make some substitutions then continue
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
        call ldlt(r,q,MX,MY,MZ)
        cc3=0.0
        do k=ksta,kend
        do j=1,my
        do i=1,mx
           cc3=cc3+r(i,j,k)*q(i,j,k)
        enddo
        enddo
        enddo
        call mpi_allreduce(cc3,c3,1,mpi_real,mpi_sum,
     &                     mpi_comm_world,ierr)
        betaiccg=c3/c1
        c1=c3
        do k=ksta,kend
        do j=1,my
        do i=1,mx
           p(i,j,k)=q(i,j,k)+betaiccg*p(i,j,k)
        enddo
        enddo
        enddo
10000 enddo 
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     fail to converge to wanted range (eps_i) 
c     after allowed iteration steps, set ier=1
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      ier=1
      itr_o=itr_i
      eps_o=res
      return
      end
