C**********************************************************************
C    induction_main.f in ,Cartesian.oct00
C
C   3-DIMENSIONAL MAGNETIC INDUCTION EQUATION SOLVER BY ICCG SCHEME
C   IN RECTANGLE CARTESIAN COORDINATES 
C
C                Last updated by Fumiko Tajima (04/17/01)
C                Original by Wenchien Chou
C
C**********************************************************************
      program induction
C**********************************************************************
C        -----   update log  -----
C   Feb. 25, 1999  reduce size of arrays in each individual CPU
C   Jan. 23, 1999  decomp.f is called only once in the main program
C   Dec.  2, 1998  MPI version
C   Sep. 29, 1998  calculate B,E field
C   Sep.  1, 1998  coded by W. Chou
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      include 'common.grid'
      include 'common.field'
      include 'common.mat'
      include 'common.para'
      include 'common.anmly'

      character*64 FNAME
      real*8 mtime2,mtime1,mtime22,mtime11
c
c    Increase the dimension of rbuf for more layers (FT, 9/13/99)
c      dimension ibuf(6),rbuf(17)  : original dimension
c       ibuf(7) one more message to pass: nlayer
c       rbuf(29) can handle up to 10 layers

      dimension ibuf(7),rbuf(29)
c     depth and sigma0 are defined in common.field
c    &    , depth(10), sigma0(10)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     initiate MPI
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      call mpi_init(ierr)
      call clocktime(mtime11)
      call mpi_comm_size(mpi_comm_world, nprocs,ierr)
      call mpi_comm_rank(mpi_comm_world, myrank,ierr)
      iup=myrank+1
      idown=myrank-1
      lastrank = nprocs-1
      if (myrank == lastrank) iup=mpi_proc_null
      if (myrank == 0 ) idown=mpi_proc_null
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     define some useful parameters
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      PAI = ACOS(-1.0)
      P2  = 2.0*PAI 
      P4  = 4.0*PAI
      P8  = 8.0*PAI
      ONEP4 = 1.0/P4
      ONEP8 = 1.0/P8
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     open and read input file INPUT
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      if (myrank ==0) then
       OPEN(UNIT=55,STATUS='OLD',FILE='INPUT',FORM='FORMATTED')
       READ(55,510) EPS
       READ(55,520) ITR
       READ(55,520) IX
       READ(55,520) JX
       READ(55,520) KXG
       READ(55,510) DX
       READ(55,510) DY
       READ(55,510) DZ
       READ(55,510) DT
       READ(55,520) NSTOP
       READ(55,515) FNAME
       READ(55,520) NSFILE
c      READ(55,520) IBLOCK
c      READ(55,520) JBLOCK
       READ(55,510) AMP
       READ(55,510) OMEGA
       READ(55,510) THETA
       READ(55,510) PHI
c
c  Addition: input the number of layers
c 
       read(55,520) nlayer
  
       do i=1,nlayer     
         read(55,510) depth(i)
       enddo
       do i=1,nlayer
         read(55,510) sigma0(i)
       enddo
c
c anomaly extent in each of 5 layers 

       read(55,510)  ext1
       read(55,510)  ext2
       read(55,510)  ext3
       read(55,510)  ext4
       read(55,510)  ext5
c
c anomaly contrast 

       read(55,510)  alph1
       read(55,510)  alph2
       read(55,510)  alph3
       read(55,510)  alph4
       read(55,510)  alph5
     
c       READ(55,510) DEPTH1
c       READ(55,510) DEPTH2
c       READ(55,510) DEPTH3
c       READ(55,510) DEPTH4
c       READ(55,510) sigma1 
c       READ(55,510) sigma2 
c       READ(55,510) sigma3 
c       READ(55,510) sigma4 
510    FORMAT(10X,G12.0)
515    FORMAT(10X,A64)
520    FORMAT(10X,I10)
530    FORMAT(10X,A1)
       CALL RMBLK(FNAME)
       CLOSE(55)
       ibuf(1)=itr
       ibuf(2)=ix
       ibuf(3)=jx
       ibuf(4)=kxg
       ibuf(5)=NSTOP
       ibuf(6)=NSFILE
       ibuf(7)=nlayer
       rbuf(1)=eps
       rbuf(2)=dx
       rbuf(3)=dy
       rbuf(4)=dz
       rbuf(5)=dt
       rbuf(6)=amp
       rbuf(7)=omega
       rbuf(8)=theta
       rbuf(9)=phi
c
c   rbuf(10)=depth1, ..., rbuf(9+n)=depthn
c
c   rbuf(10+n)=sigma1, ...., rbuf(9+2*n)=sigman
c
c       rbuf(10)=depth1
c       rbuf(11)=depth2
c       rbuf(12)=depth3
c       rbuf(13)=depth4
c       rbuf(14)=sigma1
c       rbuf(15)=sigma2
c       rbuf(16)=sigma3
c       rbuf(17)=sigma4

        do i=1,nlayer
          rbuf(9+i)=depth(i)
        enddo
      
        do i=1,nlayer
          rbuf(9+nlayer+i)=sigma0(i)
        enddo
      
      endif
      
c       nbuf=9+2*nlayer
      nbuf=29
        
      call MPI_BCAST(rbuf,nbuf,MPI_REAL,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ibuf,7,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
      
      if (myrank /= 0) then
       itr=ibuf(1)
       ix=ibuf(2)
       jx=ibuf(3)
       kxg=ibuf(4)
       nstop=ibuf(5)
       nsfile=ibuf(6)
       nlayer=ibuf(7)
       eps=rbuf(1)
       dx=rbuf(2)
       dy=rbuf(3)
       dz=rbuf(4)
       dt=rbuf(5)
       amp=rbuf(6)
       omega=rbuf(7)
       theta=rbuf(8)
       phi=rbuf(9)
       
c       depth1=rbuf(10)
c       depth2=rbuf(11)
c       depth3=rbuf(12)
c       depth4=rbuf(13)
c       sigma1=rbuf(14)
c       sigma2=rbuf(15)
c       sigma3=rbuf(16)
c       sigma4=rbuf(17)

         do i=1, nlayer
           depth(i)=rbuf(9+i)
         enddo
         
         do i=1,nlayer
           sigma0(i)=rbuf(9+nlayer+i)
         enddo
      endif
      
       THETA=(90.0-THETA)*PAI/180
       PHI=PHI*PAI/180.0
      ic=(IX+1)/2
      jc=(JX+1)/2
c     kc=(KX+1)/2
      kc=KXG
      
      write (6, 650)
  650 format(' I am before the partition')
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     partition in the k direction for each CPU, and
c     define MPI blocks for iccg: itype(irank),  amultx: ionelayer
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
       write(6,651)
  651 format('I am before para-range loop')
      do ip=0,lastrank
        call para_range(1,kxg,nprocs,ip,ista,iend)
        kksta(ip)=ista
        kkend(ip)=iend
      enddo
      kx=kkend(myrank)-kksta(myrank)+1
      ksta=1
      kend=kx
      ixx=ix+1
      jxx=jx+1
      kxx=kx+1
       write(6,652)
  652 format('I am before para-block3 loop')
      do irank=1,lastrank-1
        call para_block3(0,nx,0,ny, ixx+1, jxx+1,
     &            kkend(irank)-kksta(irank)+1, mpi_real,itype(irank))
      enddo

      write(6,653)
  653 format('I am after para_block3 loop, and line 1')
        call para_block3(0,nx,0,ny, ixx+1, jxx+1,
     &            kkend(lastrank)-kksta(lastrank)+2,
     &            mpi_real,itype(lastrank))

      write(6,654)
  654 format(' line2')
      call para_block3(0,nx,0,ny, ix,jx,1,mpi_real,ionelayer )
      write(6,655)
  655 format(' line3')
      call para_block3(0,nx,0,ny,ixx+1,jxx+1,1,mpi_real,ioneplus )

      write(6,656)
  656 format(' line4')
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     open output file, the filename was defined in INPUT file as FNAME
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
          if (myrank == 0) then
      OPEN(UNIT=2,STATUS='NEW',FILE=FNAME,FORM='UNFORMATTED')
      WRITE(2) IX,JX,KXG
      WRITE(2) DX,DY,DZ,DT,AMP,DD,PPR,OMEGA,THETA,PHI,XLENGTH
C==================================================
C     WRITE INPUT DATA
C==================================================
      WRITE(6,610)
610   FORMAT('1'///' -----------INPUT DATA----------')
C
c     WRITE(6,620) AMP,BETA,RM,GM
620   FORMAT(//' AMP=',1PE12.3,5X,'BETA=',1PE12.3,5X,'RM=',
     & 1PE12.3,5X,'GM=',1PE12.3)
C
      WRITE(6,630) DX,DY,DZ,IX,JX,KXG,DT,NSTOP,NSFILE
630   FORMAT(//' DX=',1PE12.3,5X,'DY=',1PE12.3,5X,
     &         ' DZ=',1PE12.3/' IX=',I8,5X,'JX=',I8,5X,'KX=',I8,
     &         5X,'DT=',1PE12.3,5X,'NSTOP=',I8,5X,'NSFILE=',I8)
          endif
C======================================================================
C     SET UP GRIDS AND INITIAL CONDITION  
C======================================================================
      CALL GRID
          if (myrank == 0) then
      WRITE(2) (X(I),I=0,IXX)
      WRITE(2) (Y(J),J=0,JXX)
      WRITE(2) (ZG(K),K=0,KXG+1)
          endif
      CALL INIT
      call decomp(ix,jx,kx)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c    initialize E field and save the initial vector potential A
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
       DO K=0,KXX
       DO J=0,JXX
       DO I=0,IXX
         EX(I,J,K)=0
         EY(I,J,K)=0
         EZ(I,J,K)=0
         AX0(I,J,K)=AX(I,J,K)
         AY0(I,J,K)=AY(I,J,K)
         AZ0(I,J,K)=AZ(I,J,K)
        enddo
        enddo
        enddo
      timeiccg=0.0
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
C     TIME INTEGRATION
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      DO 1000 NS=0,NSTOP
        TIME=DT*NS
        CALL PREPRHS
        call clocktime(mtime1)
        CALL iccg(prex,ax,ix,jx,kx,eps,eps_o,itr,itr_o,ier)
        
c (04/17/01)                
c        if (myrank == 0) then        
c          write(6,801) time,itr_o,eps_o
c        endif

c       CALL iccg(prey,ay,ix,jx,kx,eps,eps_o,itr,itr_o,ier)
c       if (myrank == 0) then
c         write(6,802) time,itr_o,eps_o
c       endif
c       CALL iccg(prez,az,ix,jx,kx,eps,eps_o,itr,itr_o,ier)
c       if (myrank == 0) then
c         write(6,803) time,itr_o,eps_o
c       endif
        call clocktime(mtime2)
        timeiccg=timeiccg+(mtime2-mtime1)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c    for every time step, print the number of iterations and deviation
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
801     format('ax: time= ',e12.5,' itr =',i5,'      eps= ',e12.5)
802     format('ay: time= ',e12.5,' itr =',i5,'      eps= ',e12.5)
803     format('az: time= ',e12.5,' itr =',i5,'      eps= ',e12.5)
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c   Electric field E=-dA/dt (Colomb guage and neglecting static field)
c   save ax in ax0 for use in the next time step
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
        call bnd(time)
        DO K=0,KXX
        DO J=0,JXX
        DO I=0,IXX
          EX(I,J,K) = -(AX(I,J,K)-AX0(I,J,K))/DT
          EY(I,J,K) = -(AY(I,J,K)-AY0(I,J,K))/DT
          EZ(I,J,K) = -(AZ(I,J,K)-AZ0(I,J,K))/DT
          AX0(I,J,K)=AX(I,J,K)
          AY0(I,J,K)=AY(I,J,K)
          AZ0(I,J,K)=AZ(I,J,K)
        enddo
        enddo
        enddo
        IF(MOD(NS,NSFILE).EQ.0) call file
1000  CONTINUE
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
c     print out the CPU time before ending
ccccc&cc10cccccccc20cccccccc30cccccccc40cccccccc50cccccccc60cccccccc70c
      call clocktime(mtime22 )
      total1=(mtime22-mtime11)
      write(6,*) 'myrank=',myrank,'   timeiccg=',timeiccg
      write(6,*) 'total time in myrank=',myrank,'= ', total1,'sec'
      call mpi_finalize(ierr)
      END
